# Le design pattern producteur/consommateur en python

![category](développement)
![tag](designpattern)
![tag](python)

Le principe est le suivant:

* des objets produisent des données (ou d'autres objets)
* des objets consomment les données (ou des objets) produits par les producteurs

Il n'est possible que de consommer que ce qui est produit.

ON peut limiter le nombre d'objet à consommer ou à produire à un moment t.

Tout cela est le principe de la *queue* qui peut être du type FIFO, LILO ou prioritaire

Il existe pour gérer cette queue le module *Queue* dans python.

Dans l'exemple suivant nous gérons un magasin de stroumphs

```python
import random
from Queue import *
from threading import Thread
import time

enStock = Queue(10)#notre pile, qui peut contenir 10 elements max( 0 pour enlever la limite)

class Magasin(Thread):
    """ Le 'producteur' : va remplir ses rayons de paquets de schtroumpfs"""
    def __init__(self):
        self.ouvert = True
        Thread.__init__(self)

    def run(self):
        while self.ouvert :
           
            while not enStock.full() :
                try :
                   
                    enStock.put("ajout d'un paquet de schtroumpf",True,1000)
                    ### le Magasin va ajouter un paquet dans le rayon, et s'il y a plus de place,
                    ### va attendre pendant 1 seconde qu'un paquet soit achete .
                except Full,e :
                    print( "Il n'y a pas de place pour un paquet !\n");

                else:
                    print( "On ajoute un paquet de schtroumpfs dans le rayon\n")
           
            time.sleep(10)
            #on attend 10 secondes avant que le magasin aille alimenter le rayon
       

class Client(Thread):
    """ le consommateur, va prendre un/des paquet(s) de schtroumpf"""
    def __init__(self,faim,numero):
        self.jAiFaim = faim
        self.numero = numero
        Thread.__init__(self)
       
       

    def run(self):
        while self.jAiFaim :
            time.sleep(1)
            #le client arrive au bout d'une seconde au rayon

            ok=False
            paquet = random.randint(1,4)
            #nombre de paquet qu'il va prendre
            i=1
            print( "Le client n "+str(self.numero)+ " veut "+str(paquet)+ "paquet(s) de schtroumpfs\n")
            while i<=paquet :
                try:
                    enStock.get(True,500)
                    ok = True
                    print( "n "+ str(self.numero) +  ": Prend un paquet\n")
                   
                except Empty, e:
                    ok = False
                    break
                else:
                    i+=1
            if ok :
                print( "Il reste  "+ str( enStock.qsize() ) + " paquet(s) de schtroumpfs\n")
            else :
                print( "n "+ str(self.numero) +  ": Il n'y a pas assez de paquets\n")
            self.jAiFaim = False

if __name__=='__main__':
    magasin = Magasin()
    magasin.ouvert = True
    print("##### Le magasin ouvre ses portes #####")
    magasin.start()
   
    i=1
    while i < 10:
        Client(faim = True ,numero = i).start()
        i=i+1
   
    time.sleep(60)
    magasin.ouvert=False
    print("##### Le magasin ferme ses portes #####")
```

les principales types de queues

* Queue.Queue(maxsize=0)
* Queue.LifoQueue(maxsize=0)
* Queue.PriorityQueue(maxsize=0)

les principales fonctions

* Queue.qsize()
* Queue.empty()
* Queue.full()
* Queue.put(item[, block[, timeout]])
* Queue.put_nowait(item)
* Queue.get([block[, timeout]])
* Queue.get_nowait()
* Queue.task_done()
* Queue.join()

